@@ -78,6 +78,7 @@ make_test (" #| #|\n" .
 	   "|##|\n" .
 	   "|#\n" .
 	   "  |# \"\"", "\"\"");
+make_test ("'\n(\n)", "NIL");
 make_test ("'(1 . 2)", "(1 . 2)");
 make_test ("'(1 . (2 3))", "(1 2 3)");
 make_test ("'(1 . 2\n" .
@@ -86,6 +87,7 @@ make_test ("'(1 .\n" .
 	   "2)", "(1 . 2)");
 make_test ("'(1\n" .
 	   ". 2)", "(1 . 2)");
+make_test ("'(1 .\n(2)\n)", "(1 2)");
 make_test ("'(1 . (\n" .
 	   "2))", "(1 2)");
 make_test ("'( \"\" #||# )", "(\"\")");
@@ -103,6 +105,8 @@ make_test ("'|aaa\n:bbb|", "|aaa\n:bbb|");
 make_test (":\\asd\\\\f", ":|aSD\\\\F|");
 make_test ("'cl:car", "CAR");
 make_test ("'cl-user::car", "CAR");
+make_test ("(car ''0)", "QUOTE");
+make_test ("(car '#'car)", "FUNCTION");
 
 
 # now we extensively test backtick notation.  maybe some of these
@@ -118,7 +122,8 @@ make_test ("``(a ,,(+ 1 2) ,(+ 3 4))", "`(A ,3 ,(+ 3 4))");
 make_test ("``(a ,(+ ,1 2))", "`(A ,(+ 1 2))");
 make_test ("`(,1 . ,2)", "(1 . 2)");
 make_test ("`(,1 ,2 . ,3)", "(1 2 . 3)");
-make_test ("`',(car ())", "'NIL");
+#make_test ("`',(car ())", "'NIL");
+make_test ("`',(car ())", "(QUOTE NIL)");
 make_test ("`(if ``(progn ,,1))", "(IF ``(PROGN ,,1))");
 make_test ("'@", "@");
 make_test ("`(,@())", "NIL");
@@ -280,8 +285,12 @@ make_test ("(let* ((x 1) (y x)) y)", "1");
 make_test ("(defun f ())", "F");
 make_test ("#'f", "#<FUNCTION F>");
 make_test ("#'car", "#<FUNCTION BUILTIN CAR>");
+make_test ("#'(lambda nil nil)", "#<FUNCTION ?>");
+make_test ("(flet ((foo ())) #'foo)", "#<FUNCTION ?>");
 make_test ("(function f)", "#<FUNCTION F>");
+make_test ("(flet ((foo ())) (function foo))", "#<FUNCTION ?>");
 make_test ("(function car)", "#<FUNCTION BUILTIN CAR>");
+make_test ("(function (lambda nil nil))", "#<FUNCTION ?>");
 make_test ("(apply #'+ 1 2 ())", "3");
 make_test ("(apply #'+ '(1 2))", "3");
 make_test ("(apply #'car '(1 2) ())", "1");
@@ -412,6 +421,9 @@ make_test ("(typep #p\"\" 'pathname)", "T");
 make_test ("(typep (open #p\"README\") 'stream)", "T");
 make_test ("(typep (open #p\"README\") 'file-stream)", "T");
 make_test ("(typep (make-string-input-stream \"hello\") 'string-stream)", "T");
+make_test ("(typep 0 '(or integer string))", "T");
+make_test ("(typep 0 '(and integer string))", "NIL");
+make_test ("(typep 0 '(not string))", "T");
 make_test ("(type-of 'a)", "SYMBOL");
 make_test ("(type-of \"aaa\")", "STRING");
 make_test ("(type-of #\\a)", "CHARACTER");
@@ -427,6 +439,10 @@ make_test ("(type-of (make-hash-table))", "HASH-TABLE");
 make_test ("(type-of #p\"aaa\")", "PATHNAME");
 make_test ("(type-of *standard-output*)", "STREAM");
 make_test ("(deftype not-integer () `(not integer))", "NOT-INTEGER");
+make_test ("(typep 0 'not-integer)", "NIL");
+make_test ("(typep \"\" 'not-integer)", "T");
+make_test ("(define-setf-expander foo ())", "FOO");
+make_test ("(get-setf-expansion '(foo))", "NIL");
 
 make_test ("(make-string 3)", "\"\0\0\0\"");
 make_test ("(intern \"hi\")", "|hi|\nNIL");
@@ -444,8 +460,13 @@ make_test ("(intern \"hi\" 'keyword)", ":|hi|\nNIL");
 make_test ("(make-symbol \"aaa\")", "#:|aaa|");
 make_test ("(boundp 'b)", "T");
 make_test ("(boundp 'g3)", "NIL");
+make_test ("(let ((var 0)) (boundp 'var))", "NIL");
+make_test ("(let ((var 0)) (declare (special var)) (boundp 'var))", "T");
 make_test ("(symbol-value 'b)", "9");
 make_test ("(let ((b 10)) (symbol-value 'b))", "10");
+make_test ("(setf (symbol-value 'foo) 0)", "0");
+make_test ("foo", "0");
+make_test ("(let ((b 11)) (set 'b 12) b)", "12");
 make_test ("(fboundp 'b)", "NIL");
 make_test ("(fboundp 'g3)", "T");
 make_test ("(symbol-function 'fun)", "#<FUNCTION FUN>");
@@ -465,6 +486,15 @@ make_test ("(symbol-package (make-symbol \"aaa\"))", "NIL");
 make_test ("(special-operator-p 'if)", "T");
 make_test ("(special-operator-p 'car)", "NIL");
 make_test ("(special-operator-p 'aaa)", "NIL");
+make_test ("(let ((varb 0)) (declare (special varb)) (makunbound 'varb) (boundp 'varb))", "NIL");
+make_test ("(defparameter ha 0)", "HA");
+make_test ("(boundp 'ha)", "T");
+make_test ("(makunbound 'ha)", "HA");
+make_test ("(boundp 'ha)", "NIL");
+make_test ("(defun what ())", "WHAT");
+make_test ("(fboundp 'what)", "T");
+make_test ("(fmakunbound 'what)", "WHAT");
+make_test ("(fboundp 'what)", "NIL");
 make_test ("(equal #\\a #\\a)", "T");
 make_test ("(equal 0 0)", "T");
 make_test ("(equal 0 0.0)", "NIL");
@@ -695,11 +725,11 @@ make_test ("(write 100 :stream str)", "100");
 make_test ("(write 1.2 :stream str)", "1.2");
 make_test ("(get-output-stream-string str)", "\"AAA\\\"bbb\\\"CCC#\\\\c1001.2\"");
 make_test ("(get-output-stream-string str)", "\"\"");
-make_test ("(gensym)", "#:G17");
-make_test ("(gensym)", "#:G18");
-make_test ("(gensym 5)", "#:G5");
-make_test ("(gensym \"A\")", "#:A19");
+make_test ("(gensym)", "#:G19");
 make_test ("(gensym)", "#:G20");
+make_test ("(gensym 5)", "#:G5");
+make_test ("(gensym \"A\")", "#:A21");
+make_test ("(gensym)", "#:G22");
 make_test ("(second '(0))", "NIL");
 make_test ("(fifth '(0 1 2 3 4))", "4");
 make_test ("(endp '(1 . 2))", "NIL");
@@ -707,8 +737,24 @@ make_test ("(endp ())", "T");
 make_test ("(when t \"\" \"\" \"\")", "\"\"");
 make_test ("(when nil \"\" \"\" \"\")", "NIL");
 make_test ("(unless nil \"\" \"\" \"\")", "\"\"");
+make_test ("(the integer 0)", "0");
+make_test ("(the integer (values 0 1))", "0\n1");
 make_test ("(prog1 2 (write \"hello\"))", "\"hello\"\n2");
 make_test ("(prog2 2 3 (write \"hello\"))", "\"hello\"\n3");
+make_test ("(destructuring-bind ((x)) '((0)) x)", "0");
+make_test ("(destructuring-bind (x (y (z))) '(0 (1 (2))) (list x y z))", "(0 1 2)");
+make_test ("(destructuring-bind (x (y z) w) '(0 ((1 2) 3) 4) (list x y z w))", "(0 (1 2) 3 4)");
+make_test ("(defstruct ship x y)", "SHIP");
+make_test ("#'make-ship", "#<FUNCTION MAKE-SHIP>");
+make_test ("#'ship-x", "#<FUNCTION SHIP-X>");
+make_test ("(defparameter s1 (make-ship))", "S1");
+make_test ("s1", "#<STRUCTURE OF CLASS SHIP>");
+make_test ("(ship-x s1)", "NIL");
+make_test ("(ship-y s1)", "NIL");
+make_test ("(setf (ship-x s1) 0)", "0");
+make_test ("(setf (ship-y s1) 1)", "1");
+make_test ("(ship-x s1)", "0");
+make_test ("(ship-y s1)", "1");
 make_test ("(tagbody\n" .
 	   "  (write \"1\")\n" .
 	   "  (go jmp)\n" .
@@ -724,6 +770,14 @@ make_test ("(block test (write 1) (block test2 (write 2) (return-from test (valu
 make_test ("(block nil (return (values 11 12)))", "11\n12");
 make_test ("(defun test () (write 1) (write 2) (return-from test 10) (write 3))", "TEST");
 make_test ("(test)", "12\n10");
+make_test ("(defun f () (throw 'label (values 0 1 2)))", "F");
+make_test ("(catch 'label (f))", "0\n1\n2");
+make_test ("(block nil (handler-bind ((file-error (lambda (e) (write 0))) (division-by-zero (lambda (e) (return 1)))) (/ 1 0)))", "1");
+make_test ("(block nil (handler-bind ((division-by-zero (lambda (e) (write 0))) (arithmetic-error (lambda (e) (write 1))) (error (lambda (e) (return 2)))) (/ 1 0)))", "01\n2");
+make_test ("(block nil (handler-bind ((division-by-zero (lambda (e) (return 0)))) (handler-bind ((arithmetic-error (lambda (e) (write 1)))) (/ 1 0))))", "1\n0");
+make_test ("(block test (unwind-protect 0 (write 'whatever)))", "WHATEVER\n0");
+make_test ("(block test (unwind-protect (return-from test 0) (write 'whatever)))", "WHATEVER\n0");
+make_test ("(block test (unwind-protect (return-from test (values 0 1)) (write 'whatever)))", "WHATEVER\n0\n1");
 make_test ("(cddr '(0 1 2))", "(2)");
 make_test ("(cddddr '(0 1 2 3 4))", "(4)");
 make_test ("(cadddr '(0 1 2 3 4))", "3");
@@ -888,6 +942,8 @@ make_test ("(cadr '(0 1))", "1");
 make_test ("(cl:import 'cl:car)", "T");
 make_test ("(cl:import '(cl:car) '|newtest|)", "T");
 make_test ("(cl:export nil \"newtest\")", "T");
+make_test ("(cl:export 'cdr)", "T");
+make_test ("'|newtest|:cdr", "CDR");
 make_test ("(cl:export 'jjj)", "T");
 make_test ("(cl:export '(jjj) cl:*package*)", "T");
 make_test ("'|newtest|:jjj", "JJJ");
